import fastapi
from starlette import status

from exceptions import common as common_exc, http as http_exc
from .schemas import (
    DirectorBaseSchema, DirectorListSchema, DirectorGetSchema,
    DirectorPostSchema, DirectorPatchSchema
)
from .ctrl import DirectorController


router = fastapi.APIRouter(prefix='/directors', tags=['Director'])
ctrl = DirectorController()


@router.get('')
async def get_directors(query: DirectorBaseSchema = fastapi.Depends()):
    directors = await ctrl.get_list(**query.model_dump(exclude_none=True))
    return [DirectorListSchema.from_orm(director) for director in directors]


@router.get('/{id}')
async def get_director(id: int):
    try:
        director = await ctrl.get(id)

        return DirectorGetSchema.from_orm(director)

    except common_exc.NotFoundException as e:
        raise http_exc.HTTPNotFoundException(detail=str(e))


@router.post('')
async def create_director(body: DirectorPostSchema):
    try:
        director = await ctrl.create(**body.model_dump(exclude_none=True))

        return DirectorGetSchema.from_orm(director)

    except common_exc.CreateException as e:
        raise http_exc.HTTPBadRequestException(detail=str(e))


@router.patch('/{id}')
async def update_director(id: int, body: DirectorPatchSchema):
    try:
        director = await ctrl.update(id, **body.model_dump(exclude_none=True))

        return DirectorGetSchema.from_orm(director)

    except common_exc.UpdateException as e:
        raise http_exc.HTTPBadRequestException(detail=str(e))

    except common_exc.NotFoundException as e:
        raise http_exc.HTTPNotFoundException(detail=str(e))


@router.delete('/{id}')
async def delete_director(id: int):
    try:
        await ctrl.delete(id)

        return fastapi.responses.Response(status_code=status.HTTP_204_NO_CONTENT)

    except common_exc.DeleteException as e:
        raise http_exc.HTTPBadRequestException(detail=str(e))

    except common_exc.NotFoundException as e:
        raise http_exc.HTTPNotFoundException(detail=str(e))
