import fastapi
from starlette import status

from exceptions import common as common_exc, http as http_exc
from .schemas import GenreBaseSchema, GenreGetSchema
from .ctrl import GenreController


router = fastapi.APIRouter(prefix='/genres', tags=['Genre'])
ctrl = GenreController()


@router.get('')
async def get_genres(query: GenreBaseSchema = fastapi.Depends()):
    genres = await ctrl.get_list(**query.model_dump(exclude_none=True))
    return [GenreGetSchema.from_orm(genre) for genre in genres]


@router.get('/{id}')
async def get_genre(id: int):
    try:
        director = await ctrl.get(id)

        return GenreGetSchema.from_orm(director)

    except common_exc.NotFoundException as e:
        raise http_exc.HTTPNotFoundException(detail=str(e))


@router.post('')
async def create_genre(body: GenreBaseSchema):
    try:
        director = await ctrl.create(**body.model_dump(exclude_none=True))

        return GenreGetSchema.from_orm(director)

    except common_exc.CreateException as e:
        raise http_exc.HTTPBadRequestException(detail=str(e))


@router.patch('/{id}')
async def update_genre(id: int, body: GenreBaseSchema):
    try:
        director = await ctrl.update(id, **body.model_dump(exclude_none=True))

        return GenreGetSchema.from_orm(director)

    except common_exc.UpdateException as e:
        raise http_exc.HTTPBadRequestException(detail=str(e))

    except common_exc.NotFoundException as e:
        raise http_exc.HTTPNotFoundException(detail=str(e))


@router.delete('/{id}')
async def delete_genre(id: int):
    try:
        await ctrl.delete(id)

        return fastapi.responses.Response(status_code=status.HTTP_204_NO_CONTENT)

    except common_exc.DeleteException as e:
        raise http_exc.HTTPBadRequestException(detail=str(e))

    except common_exc.NotFoundException as e:
        raise http_exc.HTTPNotFoundException(detail=str(e))
